package com.example.schedulerapp;

import java.util.concurrent.TimeUnit;

public class ModelTestSuite {
    //needs a fresh or empty database to run tests
    // If running the test again, all mySQL tables need to be truncated like below.
    /*
        SET FOREIGN_KEY_CHECKS = 0;
        truncate employees;
        truncate shifts;
        truncate positions;
        truncate timeoff;
        truncate availabilities;
     */
    public static void main(String[] args) throws InterruptedException {
        Model testModel = new Model("localhost", 8989);
        try {
            TimeUnit.SECONDS.sleep(1);
        } catch (InterruptedException exception) {
            System.out.println("Sleep");
        }
        int passed = 0;
        int total = 0;
        String output;
        Employee test;

        total += 1;
        output = testModel.addEmployee("John/", "Smith");
        if (!output.equals("Illegal character(s)")){
            System.out.println(total + ": Test addEmployee failed.");
            System.out.println("Expected: Illegal character(s)\nReceived: " + output + "\n");
        }
        else passed += 1;
        total += 1;

        output = testModel.addEmployee("John", "Smith/");
        if (!output.equals("Illegal character(s)")){
            System.out.println(total + ": Test addEmployee failed.");
            System.out.println("Expected: Illegal character(s)\nReceived: " + output + "\n");
        }
        else passed += 1;
        total += 1;

        output = testModel.addEmployee("John,", "Smith");
        if (!output.equals("Illegal character(s)")){
            System.out.println(total + ": Test addEmployee failed.");
            System.out.println("Expected: Illegal character(s)\nReceived: " + output + "\n");
        }
        else passed += 1;
        total += 1;

        output = testModel.addEmployee("John,", "Smith,");
        if (!output.equals("Illegal character(s)")){
            System.out.println(total + ": Test addEmployee failed.");
            System.out.println("Expected: Illegal character(s)\nReceived: " + output + "\n");
        }
        else passed += 1;
        total += 1;

        output = testModel.removeEmployee("-1");
        if (!output.equals("Invalid ID")){
            System.out.println(total + ": Test removeEmployee failed.");
            System.out.println("Expected: Invalid IDt\nReceived: " + output + "\n");
        }
        else passed += 1;
        total += 1;

        output = testModel.removeEmployee("1");
        if (!output.equals("Employee does not exist")){
            System.out.println(total + ": Test removeEmployee failed.");
            System.out.println("Expected: Employee does not exist\nReceived: " + output + "\n");
        }
        else passed += 1;
        total += 1;

        output = testModel.removeEmployee("1");
        if (!output.equals("Employee does not exist")){
            System.out.println(total + ": Test removeEmployee failed.");
            System.out.println("Expected: Employee does not exist\nReceived: " + output + "\n");
        }
        else passed += 1;
        total += 1;

        output = testModel.removeEmployee("-1");
        if (!output.equals("Invalid ID")){
            System.out.println(total + ": Test removeEmployee failed.");
            System.out.println("Expected: Invalid IDt\nReceived: " + output + "\n");
        }
        else passed += 1;
        total += 1;

        output = testModel.addShift(1, "2022-03-20", 900, 1700);
        if (!output.equals("No Employee with that ID")){
            System.out.println(total + ": Test addShift failed.");
            System.out.println("Expected: No Employee with that ID\nReceived: " + output + "\n");
        }
        else passed += 1;
        total += 1;

        output = testModel.addShift(-1, "2022-03-20", 900, 1700);
        if (!output.equals("Invalid ID")){
            System.out.println(total + ": Test addShift failed.");
            System.out.println("Expected: Invalid ID\nReceived: " + output + "\n");
        }
        else passed += 1;
        total += 1;

        output = testModel.addShift(1, "2022-03-20", -1, 1700);
        if (!output.equals("Invalid start or end time")){
            System.out.println(total + ": Test addShift failed.");
            System.out.println("Invalid start or end time\nReceived: " + output + "\n");
        }
        else passed += 1;
        total += 1;

        output = testModel.addShift(1, "2022-03-20", 900, -1);
        if (!output.equals("Invalid start or end time")){
            System.out.println(total + ": Test addShift failed.");
            System.out.println("Invalid start or end time\nReceived: " + output + "\n");
        }
        else passed += 1;
        total += 1;

        output = testModel.addShift(1, "2022-03-20", 2400, 1700);
        if (!output.equals("Invalid start or end time")){
            System.out.println(total + ": Test addShift failed.");
            System.out.println("Invalid start or end time\nReceived: " + output + "\n");
        }
        else passed += 1;
        total += 1;

        output = testModel.addShift(1, "2022-03-20", 900, 2400);
        if (!output.equals("Invalid start or end time")){
            System.out.println(total + ": Test addShift failed.");
            System.out.println("Invalid start or end time\nReceived: " + output + "\n");
        }
        else passed += 1;
        total += 1;

        output = testModel.addEmployee("John", "Smith");
        if (!output.equals("Employee added")){
            System.out.println(total + ": Test addEmployee failed.");
            System.out.println("Expected: Employee added\nReceived: " + output + "\n");
        }
        else passed += 1;
        total += 1;

        TimeUnit.SECONDS.sleep(1);
        output = testModel.getEmployee(testModel.getIDbyIndex(0)).getFullName();
        if (!output.equals("John Smith")){
            System.out.println(total + ": Test getEmployeeByID failed.");
            System.out.println("Expected: John Smith\nReceived: " + output + "\n");
        }
        else passed += 1;
        total += 1;

        TimeUnit.SECONDS.sleep(1);
        output = testModel.getEmployeeByID(1);
        if (!output.equals("John Smith")){
            System.out.println(total + ": Test getEmployee failed.");
            System.out.println("Expected: John Smith\nReceived: " + output + "\n");
        }
        else passed += 1;
        total += 1;

        if (testModel.employeeSize() != 1){
            System.out.println(total + ": Test employeeSize failed.");
            System.out.println("Expected: 1\nReceived: " + testModel.employeeSize() + "\n");
        }
        else passed += 1;
        total += 1;

        output = testModel.addEmployee("Freya", "Fulltime");
        if (!output.equals("Employee added")){
            System.out.println(total + ": Test addEmployee failed.");
            System.out.println("Expected: Employee added\nReceived: " + output + "\n");
        }
        else passed += 1;
        total += 1;

        TimeUnit.SECONDS.sleep(1);
        output = testModel.getEmployee(testModel.getIDbyIndex(1)).getFullName();
        if (!output.equals("Freya Fulltime")){
            System.out.println(total + ": Test getEmployee failed.");
            System.out.println("Expected: Freya Fulltime\nReceived: " + output + "\n");
        }
        else passed += 1;
        total += 1;

        TimeUnit.SECONDS.sleep(1);
        output = testModel.getEmployeeByID(2);
        if (!output.equals("Freya Fulltime")){
            System.out.println(total + ": Test getEmployeeByID failed.");
            System.out.println("Expected: Freya Fulltime\nReceived: " + output + "\n");
        }
        else passed += 1;
        total += 1;

        if (testModel.employeeSize() != 2){
            System.out.println(total + ": Test employeeSize failed.");
            System.out.println("Expected: 2\nReceived: " + testModel.employeeSize() + "\n");
        }
        else passed += 1;
        total += 1;

        output = testModel.addShift(1, "2022-03-20", 900, 1700);
        if (!output.equals("New shift added")){
            System.out.println(total + ": Test addShift failed.");
            System.out.println("Expected: New shift added\nReceived: " + output + "\n");
        }
        else passed += 1;
        total += 1;

        TimeUnit.SECONDS.sleep(1);
        output = testModel.addShift(1, "2022-03-20", 900, 1700);
        if (!output.equals("Shift time conflict for employee")){
            System.out.println(total + ": Test addShift failed.");
            System.out.println("Expected: Shift time conflict for employee\nReceived: " + output + "\n");
        }
        else passed += 1;
        total += 1;

        TimeUnit.SECONDS.sleep(1);
        output = testModel.addShift(1, "2022-03-20", 0, 800);
        if (!output.equals("Shift time conflict for employee")){
            System.out.println(total + ": Test addShift failed.");
            System.out.println("Expected: Shift time conflict for employee\nReceived: " + output + "\n");
        }
        else passed += 1;
        total += 1;

        TimeUnit.SECONDS.sleep(1);
        output = testModel.addShift(1, "2022-03-20", 1800, 2359);
        if (!output.equals("Shift time conflict for employee")){
            System.out.println(total + ": Test addShift failed.");
            System.out.println("Expected: Shift time conflict for employee\nReceived: " + output + "\n");
        }
        else passed += 1;
        total += 1;

        output = testModel.addShift(1, "2022-03-21", 900, 1700);
        if (!output.equals("New shift added")){
            System.out.println(total + ": Test addShift failed.");
            System.out.println("Expected: New shift added\nReceived: " + output + "\n");
        }
        else passed += 1;
        total += 1;

        output = testModel.addShift(2, "2022-03-20", 900, 1700);
        if (!output.equals("New shift added")){
            System.out.println(total + ": Test addShift failed.");
            System.out.println("Expected: New shift added\nReceived: " + output + "\n");
        }
        else passed += 1;
        total += 1;

        output = testModel.editEmployee(1, "John", "Smith", false, "Johnsmith@gmail.com", "306-123-4567", -12.0f);
        if (!output.equals("Invalid wage")){
            System.out.println(total + ": Test editEmployee failed.");
            System.out.println("Expected: Invalid wage\nReceived: " + output + "\n");
        }
        else passed += 1;
        total += 1;

        output = testModel.editEmployee(1, "John", "Smith", false, "Johnsmith@gmail.com", "306/123-4567", 12.0f);
        if (!output.equals("phoneNumber contains illegal character(s)")){
            System.out.println(total + ": Test editEmployee failed.");
            System.out.println("Expected: phoneNumber contains illegal character(s)\nReceived: " + output + "\n");
        }
        else passed += 1;
        total += 1;

        output = testModel.editEmployee(1, "John", "Smith", false, "Johnsmith@gmail.com", "306-123,4567", 12.0f);
        if (!output.equals("phoneNumber contains illegal character(s)")){
            System.out.println(total + ": Test editEmployee failed.");
            System.out.println("Expected: phoneNumber contains illegal character(s)\nReceived: " + output + "\n");
        }
        else passed += 1;
        total += 1;

        output = testModel.editEmployee(1, "John", "Smith", false, "Johnsmith@gmail/.com", "306-123-4567", 12.0f);
        if (!output.equals("Email contains illegal character(s)")){
            System.out.println(total + ": Test editEmployee failed.");
            System.out.println("Expected: Email contains illegal character(s)\nReceived: " + output + "\n");
        }
        else passed += 1;
        total += 1;

        output = testModel.editEmployee(1, "John", "Smith", false, "Johnsmith@gmail.com,", "306-123-4567", 12.0f);
        if (!output.equals("Email contains illegal character(s)")){
            System.out.println(total + ": Test editEmployee failed.");
            System.out.println("Expected: Email contains illegal character(s)\nReceived: " + output + "\n");
        }
        else passed += 1;
        total += 1;

        output = testModel.editEmployee(1, "John", "Smi/th", false, "Johnsmith@gmail.com", "306-123-4567", 12.0f);
        if (!output.equals("Name contains illegal character(s)")){
            System.out.println(total + ": Test editEmployee failed.");
            System.out.println("Expected: Name contains illegal character(s)\nReceived: " + output + "\n");
        }
        else passed += 1;
        total += 1;

        output = testModel.editEmployee(1, "J,ohn", "Smith", false, "Johnsmith@gmail.com", "306-123-4567", 12.0f);
        if (!output.equals("Name contains illegal character(s)")){
            System.out.println(total + ": Test editEmployee failed.");
            System.out.println("Expected: Name contains illegal character(s)\nReceived: " + output + "\n");
        }
        else passed += 1;
        total += 1;

        output = testModel.editEmployee(-1, "John", "Smith", false, "Johnsmith@gmail.com", "306-123-4567", 12.0f);
        if (!output.equals("Invalid ID")){
            System.out.println(total + ": Test editEmployee failed.");
            System.out.println("Expected: Invalid ID\nReceived: " + output + "\n");
        }
        else passed += 1;
        total += 1;

        output = testModel.editEmployee(4, "John", "Smith", false, "Johnsmith@gmail.com", "306-123-4567", 12.0f);
        if (!output.equals("Employee does not exist")){
            System.out.println(total + ": Test editEmployee failed.");
            System.out.println("Expected: Employee does not exist\nReceived: " + output + "\n");
        }
        else passed += 1;
        total += 1;

        output = testModel.editEmployee(1, "John", "Smith", false, "Johnsmith@gmail.com", "306-123-4567", 12.0f);
        if (!output.equals("Employee edited")){
            System.out.println(total + ": Test editEmployee failed.");
            System.out.println("Expected: Employee edited\nReceived: " + output + "\n");
        }
        else passed += 1;
        total += 1;

        output = testModel.editEmployee(2, "John", "Smith", false, "Johnsmith@gmail.com", "306-123-4567", 12.0f);
        if (!output.equals("Employee edited")){
            System.out.println(total + ": Test editEmployee failed.");
            System.out.println("Expected: Employee edited\nReceived: " + output + "\n");
        }
        else passed += 1;
        total += 1;

        /*
        TimeUnit.SECONDS.sleep(1);
        output = testModel.removeEmployee("2");
        if (!output.equals("Employee removed")){
            System.out.println(total + ": Test removeEmployee failed.");
            System.out.println("Expected: Employee removed\nReceived: " + output + "\n");
        }
        else passed += 1;
        total += 1;

        TimeUnit.SECONDS.sleep(1);
        if (testModel.employeeSize() != 1){
            System.out.println(total + ": Test employeeSize failed.");
            System.out.println("Expected: 1\nReceived: " + testModel.employeeSize() + "\n");
        }
        else passed += 1;
        total += 1;

        output = testModel.removeEmployee("1");
        if (!output.equals("Employee removed")){
            System.out.println(total + ": Test removeEmployee failed.");
            System.out.println("Expected: Employee does not exist\nReceived: " + output + "\n");
        }
        else passed += 1;
        total += 1;

        TimeUnit.SECONDS.sleep(1);
        output = testModel.removeEmployee("1");
        if (!output.equals("Employee does not exist")){
            System.out.println(total + ": Test removeEmployee failed.");
            System.out.println("Expected: Employee removed\nReceived: " + output + "\n");
        }
        else passed += 1;
        total += 1;


         */
        total-= 1;
        System.out.println(passed + "/" + total + " tests passed.");
        System.exit(0);
    }
}