package com.example.schedulerapp;

import javafx.application.Platform;
import javafx.collections.FXCollections;
import javafx.collections.ObservableList;
import javafx.event.ActionEvent;
import javafx.event.EventHandler;
import javafx.geometry.Insets;
import javafx.scene.Node;
import javafx.scene.control.Button;
import javafx.scene.control.ComboBox;
import javafx.scene.control.Label;
import javafx.scene.control.ScrollPane;
import javafx.scene.layout.GridPane;
import javafx.scene.layout.VBox;
import javafx.stage.Popup;

import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.Timer;
import java.util.TimerTask;

public class ScheduleView extends ScrollPane implements ModelSubscriber{
    Model model;
    ArrayList<Label> times;
    ArrayList<String> timesString;
    Popup dropShift;
    Popup editShiftPopup;
    Popup reassignPopup;
    DateTimeFormatter dateFormatView;
    DateTimeFormatter timeFormat;
    DateTimeFormatter dbTimeFormat;
    GridPane contents;


    // must be implemented by the child class
    public void draw(){}

    public ScheduleView(){
        contents = new GridPane();
        this.setContent(contents);

        //list of time labels
        times = new ArrayList<>();
        timesString = new ArrayList<>();
        for (int i = 8; i < 24; i++){
            String j = i + "00";
            String k = i + "30";
            if (i < 10) {
                j = "0" + j;
                k = "0" + k;
            }
            Label jLabel = new Label(j);
            Label kLabel = new Label(k);
            times.add(jLabel);
            times.add(kLabel);
            timesString.add(j);
            timesString.add(k);
        }
        for (Label label : times){
            label.setPrefHeight(18);
            //label.setStyle(" -fx-border-color: black");
        }
        dateFormatView = DateTimeFormatter.ofPattern("MMM dd, yyyy");
        timeFormat = DateTimeFormatter.ofPattern("kk:mm");
        dbTimeFormat = DateTimeFormatter.ofPattern("kkmm");
    }

    public void setModel(Model newModel) {
        this.model = newModel;
        //now can draw for first time
        Timer drawTimer = new Timer(true);
        drawTimer.scheduleAtFixedRate(new TimerTask() {
            @Override
            public void run() {
                Platform.runLater(() -> {
                    draw();
                });
            }
        }, 1000, 1000);
    }

    public void drawPopUp(Shift shift){
        dropShift = new Popup();
        dropShift.setHeight(300);
        dropShift.setWidth(300);

        VBox details = new VBox();
        details.setStyle("-fx-background-color: papayawhip; -fx-border-color: black");
        details.setPadding(new Insets(20));
        details.setSpacing(20);

        Label name = new Label("Name: " + model.getEmployeeByID(shift.getEmployeeID()));
        Label position = new Label("Position: " + shift.getPosition());
        Label date = new Label("Date: " + shift.getStart().format(dateFormatView));
        Label start = new Label("Start: " + shift.getStart().format(timeFormat));
        Label end = new Label("End: " + shift.getEnd().format(timeFormat));
        Label result = new Label();

        GridPane buttons = new GridPane();
        buttons.setHgap(10);
        buttons.setVgap(10);

        //Buttons
        Button cancel = new Button("Cancel");
        cancel.setCancelButton(true);
        cancel.setOnAction(cancelDrop);   //debug

        Button drop = new Button("Drop Shift");
        drop.setOnAction(e -> {
            dropShift(shift.getShiftID());
            result.setText("Shift dropped");
        });

        //Manager Buttons
        Button remove = new Button("Remove Shift");
        remove.setOnAction(e -> removeShift(shift.getShiftID()));

        Button edit = new Button("Edit Shift");
        edit.setOnAction(e -> {
            drawEditPopUp(shift);
            editShiftPopup.show(this.getScene().getWindow());
        });

        Button reassign = new Button("Reassign Shift");
        reassign.setOnAction(e -> {
            drawReassignPopUp(shift);
            reassignPopup.show(this.getScene().getWindow());
        });

        buttons.add(cancel, 0, 0);
        if (model.getThisEmployee() == shift.getEmployeeID()){
            buttons.add(drop, 1, 0);
        }
        if (model.getIsManager()) {
            buttons.add(edit, 0, 1);
            buttons.add(remove, 1, 1);
            buttons.add(reassign, 2, 1);
        }
        for (Node button : buttons.getChildren()){
            ((Button) button).setMaxWidth(200);
        }

        details.getChildren().addAll(name, position, date, start, end, result, buttons);
        dropShift.getContent().add(details);
        dropShift.setAutoHide(true);
    }

    //for changing time
    public void drawEditPopUp(Shift shift){
        editShiftPopup = new Popup();
        editShiftPopup.setHeight(300);
        editShiftPopup.setWidth(300);

        VBox details = new VBox();
        details.setStyle("-fx-background-color: papayawhip; -fx-border-color: black");
        details.setPadding(new Insets(20));
        details.setSpacing(20);

        Label name = new Label("Name: " + model.getEmployeeByID(shift.getEmployeeID()));
        Label position = new Label("Position: " + shift.getPosition());
        Label date = new Label("Date: " + shift.getStart().format(dateFormatView));
        ComboBox<String> start = new ComboBox<>(FXCollections.observableArrayList(timesString));
        ComboBox<String> end = new ComboBox<>(FXCollections.observableArrayList(timesString));
        Label result = new Label();

        GridPane buttons = new GridPane();
        buttons.setHgap(10);
        buttons.setVgap(10);

        //Buttons
        Button cancel = new Button("Close");
        cancel.setCancelButton(true);
        cancel.setOnAction(e -> cancelEdit());

        Button confirm = new Button("Confirm");
        confirm.setOnAction(e -> {
            result.setText(editShift(shift.getShiftID(), start.getValue(), end.getValue()));
        });

        buttons.addRow(0, cancel, confirm);

        details.getChildren().addAll(name, position, date, start, end, result, buttons);
        editShiftPopup.getContent().add(details);
        editShiftPopup.setAutoHide(true);
    }

    public void drawReassignPopUp(Shift shift){
        reassignPopup = new Popup();
        reassignPopup.setHeight(300);
        reassignPopup.setWidth(300);

        VBox details = new VBox();
        details.setStyle("-fx-background-color: papayawhip; -fx-border-color: black");
        details.setPadding(new Insets(20));
        details.setSpacing(20);

        ComboBox<Employee> name = new ComboBox<>(FXCollections.observableArrayList(model.getAllEmployees()));
        Label position = new Label("Position: " + shift.getPosition());
        Label date = new Label("Date: " + shift.getStart().format(dateFormatView));
        Label start = new Label("Start: " + shift.getStart().format(timeFormat));
        Label end = new Label("End: " + shift.getEnd().format(timeFormat));
        Label result = new Label();

        GridPane buttons = new GridPane();
        buttons.setHgap(10);
        buttons.setVgap(10);

        //Buttons
        Button cancel = new Button("Close");
        cancel.setCancelButton(true);
        cancel.setOnAction(e -> cancelReassign());

        Button confirm = new Button("Confirm");
        confirm.setOnAction(e -> {
            result.setText(reassignShift(shift.getShiftID(), name.getSelectionModel().getSelectedItem().getEmployeeID()));
        });

        buttons.addRow(0, cancel, confirm);

        details.getChildren().addAll(name, position, date, start, end, result, buttons);
        reassignPopup.getContent().add(details);
        reassignPopup.setAutoHide(true);
    }



    public void modelChanged() {
        draw();
    }

    public String formatShiftTimeLabel(Shift shift){
        String text = shift.getStart().format(timeFormat);
        text += "\n   -   \n";
        text += shift.getEnd().format(timeFormat);
        return text;
    }

    public ShiftButton formatShiftButton(Shift shift){
        ShiftButton shiftRep = new ShiftButton(formatShiftTimeLabel(shift), shift);
        shiftRep.setMaxHeight(700);                 //so it always fills its region
        shiftRep.setMaxWidth(100);
        GridPane.setFillHeight(shiftRep, true);
        GridPane.setFillWidth(shiftRep, true);
        //css styling
        //todo: different colour for current user
        shiftRep.setStyle("-fx-background-color: skyblue");

        shiftRep.setOnAction(dropShiftPopUp);

        return shiftRep;
    }

    public int shiftGetRow(Shift shift){
        //convert from start time to offset, granularity in 30 min increments
        int row = (shift.getStart().getHour() - 8 ) * 2 + 1;
        if (shift.getStart().getMinute() == 30) row++;
        return row;
    }

    public int shiftGetLength(Shift shift){
        //calculate length
        int length = (shift.getEnd().getHour() - shift.getStart().getHour()) * 2;
        if (shift.getStart().getMinute() == 30) length--;
        if (shift.getEnd().getMinute() == 30) length++;
        return length;
    }

    //event handlers
    EventHandler<ActionEvent> dropShiftPopUp =
            new EventHandler<>() {
                @Override
                public void handle(ActionEvent e) {
                    drawPopUp(((ShiftButton) e.getSource()).getShift());
                    dropShift.show(((Node) e.getSource()).getScene().getWindow());
                }
            };

    EventHandler<ActionEvent> dropShiftConfirm =
            new EventHandler<>() {
                @Override
                public void handle(ActionEvent e) {
                    System.out.println("todo: drop shift :)");
                    dropShift.hide();
                }
            };

    EventHandler<ActionEvent> cancelDrop =
            new EventHandler<>() {
                @Override
                public void handle(ActionEvent actionEvent) {
                    System.out.println("Shift drop cancelled.");
                    dropShift.hide();
                }
            };



    private void removeShift(int shiftID){
        model.removeShift(shiftID);
        dropShift.hide();
    }

    private String editShift(int shiftID, String start, String end){
        Shift shift = model.getShift(shiftID);
        return model.editShift(shiftID, shift.getDate(), Integer.parseInt(start), Integer.parseInt(end),
                shift.isAvailable(), shift.getEmployeeID());
    }

    private String reassignShift(int shiftID, int empID){
        Shift shift = model.getShift(shiftID);
        return model.editShift(shiftID, shift.getDate(), Integer.parseInt(shift.getStart().format(dbTimeFormat)),
                Integer.parseInt(shift.getEnd().format(dbTimeFormat)), false, empID);
    }

    private void cancelEdit(){
        editShiftPopup.hide();
    }

    private void cancelReassign(){
        reassignPopup.hide();
    }

    private void dropShift(int shiftID){
        model.editShiftAvailability(shiftID, true);
    }
}
